@@ -37,8 +37,9 @@
 #ifndef LIBTORRENT_OBJECT_H
 #define LIBTORRENT_OBJECT_H
 
-#include <string>
+#include <limits>
 #include <map>
+#include <string>
 #include <vector>
 #include <torrent/common.h>
 #include <torrent/exceptions.h>
@@ -48,25 +49,25 @@ namespace torrent {
 
 class LIBTORRENT_EXPORT Object {
 public:
-  typedef int64_t                           value_type;
-  typedef std::string                       string_type;
-  typedef std::vector<Object>               list_type;
-  typedef std::map<std::string, Object>     map_type;
-  typedef map_type*                         map_ptr_type;
-  typedef map_type::key_type                key_type;
-  typedef std::pair<std::string, Object*>   dict_key_type;
+  using value_type    = int64_t;
+  using string_type   = std::string;
+  using list_type     = std::vector<Object>;
+  using map_type      = std::map<std::string, Object>;
+  using map_ptr_type  = map_type*;
+  using key_type      = map_type::key_type;
+  using dict_key_type = std::pair<std::string, Object*>;
 
-  typedef list_type::iterator               list_iterator;
-  typedef list_type::const_iterator         list_const_iterator;
-  typedef list_type::reverse_iterator       list_reverse_iterator;
-  typedef list_type::const_reverse_iterator list_const_reverse_iterator;
+  using list_iterator               = list_type::iterator;
+  using list_const_iterator         = list_type::const_iterator;
+  using list_reverse_iterator       = list_type::reverse_iterator;
+  using list_const_reverse_iterator = list_type::const_reverse_iterator;
 
-  typedef map_type::iterator                map_iterator;
-  typedef map_type::const_iterator          map_const_iterator;
-  typedef map_type::reverse_iterator        map_reverse_iterator;
-  typedef map_type::const_reverse_iterator  map_const_reverse_iterator;
+  using map_iterator               = map_type::iterator;
+  using map_const_iterator         = map_type::const_iterator;
+  using map_reverse_iterator       = map_type::reverse_iterator;
+  using map_const_reverse_iterator = map_type::const_reverse_iterator;
 
-  typedef std::pair<map_iterator, bool>     map_insert_type;
+  using map_insert_type = std::pair<map_iterator, bool>;
 
   // Flags in the range of 0xffff0000 may be set by the user, however
   // 0x00ff0000 are reserved for keywords defined by libtorrent.
@@ -97,7 +98,6 @@ public:
     TYPE_DICT_KEY
   };
 
-  Object()                     : m_flags(TYPE_NONE) {}
   Object(const value_type v)   : m_flags(TYPE_VALUE) { new (&_value()) value_type(v); }
   Object(const char* s)        : m_flags(TYPE_STRING) { new (&_string()) string_type(s); }
   Object(const string_type& s) : m_flags(TYPE_STRING) { new (&_string()) string_type(s); }
@@ -105,9 +105,11 @@ public:
   Object(const raw_string& r)  : m_flags(TYPE_RAW_STRING) { new (&_raw_string()) raw_string(r); }
   Object(const raw_list& r)    : m_flags(TYPE_RAW_LIST) { new (&_raw_list()) raw_list(r); }
   Object(const raw_map& r)     : m_flags(TYPE_RAW_MAP) { new (&_raw_map()) raw_map(r); }
-  Object(const Object& b);
 
+  Object() {}
   ~Object() { clear(); }
+  Object(const Object& b);
+  Object& operator=(const Object& b);
 
   // TODO: Move this out of the class namespace, call them
   // make_object_. 
@@ -162,6 +164,7 @@ public:
   string_type&        as_string()                             { check_throw(TYPE_STRING); return _string(); }
   const string_type&  as_string() const                       { check_throw(TYPE_STRING); return _string(); }
   const string_type&  as_string_c() const                     { check_throw(TYPE_STRING); return _string(); }
+  const char*         as_c_str() const                        { check_throw(TYPE_STRING); return _string().c_str(); }
   list_type&          as_list()                               { check_throw(TYPE_LIST); return _list(); }
   const list_type&    as_list() const                         { check_throw(TYPE_LIST); return _list(); }
   map_type&           as_map()                                { check_throw(TYPE_MAP); return _map(); }
@@ -179,6 +182,8 @@ public:
   raw_map&            as_raw_map()                            { check_throw(TYPE_RAW_MAP); return _raw_map(); }
   const raw_map&      as_raw_map() const                      { check_throw(TYPE_RAW_MAP); return _raw_map(); }
 
+  template <typename T> T as_value_type(const char* err_msg) const { check_value_throw<T>(err_msg); return _value(); }
+
   bool                has_key(const key_type& k) const        { check_throw(TYPE_MAP); return _map().find(k) != _map().end(); }
   bool                has_key_value(const key_type& k) const  { check_throw(TYPE_MAP); return check(_map().find(k), TYPE_VALUE); }
   bool                has_key_string(const key_type& k) const { check_throw(TYPE_MAP); return check(_map().find(k), TYPE_STRING); }
@@ -237,8 +242,6 @@ public:
                                  uint32_t skip_mask = flag_static_data,
                                  uint32_t maxDepth = ~uint32_t());
 
-  Object&             operator = (const Object& b);
-
   // Internal:
   void                swap_same_type(Object& left, Object& right);
 
@@ -246,9 +249,11 @@ public:
   inline bool         check(map_type::const_iterator itr, type_type t) const { return itr != _map().end() && itr->second.type() == t; }
   inline void         check_throw(type_type t) const                         { if (t != type()) throw bencode_error("Wrong object type."); }
 
-  uint32_t            m_flags;
+  template <typename T> void check_value_throw(const char* err_msg) const;
 
-#ifdef HAVE_CXX11
+  uint32_t            m_flags{TYPE_NONE};
+
+#ifndef HAVE_STDCXX_0X
   value_type&         _value()             { return t_value; }
   const value_type&   _value() const       { return t_value; }
   string_type&        _string()            { return t_string; }
@@ -484,6 +489,19 @@ object_equal(const Object& left, const Object& right) {
   }
 }
 
+template <typename T>
+inline void
+Object::check_value_throw(const char* err_msg) const {
+  if (!std::numeric_limits<T>::is_integer)
+    throw internal_error("Tried to check value with non-integer type.");
+
+  if (!is_value())
+    throw bencode_error(err_msg);
+
+  if (!(_value() >= std::numeric_limits<T>::min() && _value() <= std::numeric_limits<T>::max()))
+    throw bencode_error(err_msg);
+}
+
 }
 
 #endif
